import org.jfugue.*;

//------------------------------ Class::Beat --------------------------------
public class Beat {

    public Pattern pattern = new Pattern();
    Chromosome rChrom;
    int numBeats;
    char beatType;
    int[] instrumentation = new int[2];

    private String generateRidePart() {
        //Generate hi-hat string
        String hat = "^";
        for (int i=1; i<numBeats; i++) {
            if(numBeats>3 && (rChrom.getNextAlleleValue()%4)==1) hat += "^";
            else hat += ".";
        }
        return hat;
    }
    
    private String generateLowAccentPart() {
        String bassDrum = "O";
        for (int i=1; i<numBeats; i++) {
            if(i==2 && (rChrom.getNextAlleleValue()%2)==1) {
                bassDrum += "O";
            } else {
                bassDrum += ".";
            }
        }
        return bassDrum;
    }

    private String generateHighAccentPart() {
        String snare = new String();
        //Generate snare string
        if (beatType=='h') {
            snare = ".*";
            for (int i=2; i<numBeats; i++)
                if(i%2==0) snare += ".*";
        }//if beat is measured in half notes
        if ( beatType=='q' && numBeats!=3) {
            snare = "..*";
            for (int i=3; i<numBeats; i++) {
                if (i==6 || i==10) snare += "*";
                else snare += ".";
            }
        }//if beat is measured in quarter notes and isn't in 3/4
        if ( (beatType=='q' || beatType=='s') && numBeats==3) {
            snare = ".**";
            for (int i=3; i<numBeats; i++) {
                if (i==3 || i==6 || i==9) snare += ".**";
            }
        }//if beat is in 3/4
        if (beatType=='i') {
            if(numBeats==6) snare = "...*.";
            else snare = "....*";
            for (int i=5; i<numBeats; i++) {
                if (i==9) snare += "*";
                else snare += ".";
            }
        }//if beat is measured in eighth notes
        return snare;
    }

    Pattern generateBeat() {
        Pattern myPattern;
        String snare = new String();
        org.jfugue.Rhythm beat = new org.jfugue.Rhythm();

        String hat = generateRidePart();

        //Generate bassDrum string
        String bassDrum = generateLowAccentPart();

        snare = generateHighAccentPart();

        //Add strings to the beat and generate pattern
        beat.setLayer(1, bassDrum);
        beat.setLayer(2, snare);
        beat.setLayer(3, hat);

       

        if (instrumentation[0]==0) {
            beat.addSubstitution('O', "[LOW_BONGO]" + Character.toString(beatType));
            beat.addSubstitution('*', "[HIGH_TIMBALE]" + Character.toString(beatType));
            beat.addSubstitution('^', "[HI_BONGO]" +  Character.toString(beatType));
        } else if(instrumentation[0]==1) {
            beat.addSubstitution('O', "[LOW_CONGA]" + Character.toString(beatType));
            beat.addSubstitution('*', "[OPEN_HI_CONGA]" + Character.toString(beatType));
            beat.addSubstitution('^', "[MUTE_HI_CONGA]" +  Character.toString(beatType));

        } else if(instrumentation[0]==2) {
            beat.addSubstitution('O', "[LOW_TIMBALE]" + Character.toString(beatType));
            beat.addSubstitution('*', "[HIGH_TIMBALE]" + Character.toString(beatType));
            if (instrumentation[1]==1) beat.addSubstitution('^', "[SIDE_STICK]" +  Character.toString(beatType));
            else beat.addSubstitution('^', "[COWBELL]" +  Character.toString(beatType));
        } else if(instrumentation[0]==3) {
            beat.addSubstitution('O', "[ACOUSTIC_BASS_DRUM]" + Character.toString(beatType));
            beat.addSubstitution('*', "[ACOUSTIC_SNARE]" + Character.toString(beatType));
            if (instrumentation[1]==1)
                beat.addSubstitution('^', "[RIDE_CYMBAL_1]" +  Character.toString(beatType));
            else
                beat.addSubstitution('^', "[RIDE_CYMBAL_2]" +  Character.toString(beatType));
        } else {
            beat.addSubstitution('O', "[BASS_DRUM]" + Character.toString(beatType));
            beat.addSubstitution('*', "[ELECTRIC_SNARE]" + Character.toString(beatType));
            if (instrumentation[1]==1)
                beat.addSubstitution('^', "[CLOSED_HI_HAT]" +  Character.toString(beatType));
            else
                beat.addSubstitution('^', "[OPEN_HI_HAT]" +  Character.toString(beatType));
        }

        beat.addSubstitution('.', "R" +  Character.toString(beatType));
        myPattern = beat.getPattern();
        return myPattern;

    }

    //----------------------------- Constructor -------------------------------
    //  Generates a beat and initializes its pattern
    //  Parameters:  Type of Beat, Number of Beats, Number of Chords,
    //               Number of Times to Loop Chord Progression, Rhythm Chromosome
    //-------------------------------------------------------------------------
    public Beat(char beatTypePR, int numBeatsPR, int numChordsPR, int numChordLoopsPR, Chromosome rChromPR) {
        numBeats = numBeatsPR;
        beatType = beatTypePR;
        rChrom = rChromPR;
        instrumentation[0] = (rChrom.getNextAlleleValue()%4);
        instrumentation[1] = (rChrom.getNextAlleleValue()%2);
        //pattern = generateBeat();
        pattern.add(generateBeat());
        pattern.add(generateBeat());
        pattern.repeat((numChordsPR*numChordLoopsPR)/2);
    }//beat()
}
