import org.jfugue.*;
import java.io.*;
import java.util.*;

//----------------------------- Class::Generator --------------------------------
public class Generator {

  //Time signature data
  private char beatType;
  private int numBeats = -1;

  //Amount to transpose song, number of melodies, and tempo
  private int trans; 
  private int numMelodies; 
  private String tempo = new String();

  //Voices for the various parts of the song
  private String bassVoice = new String();
  private String rythymVoice = new String();
  private String[] leadVoices = new String[3];
  
  //Song's genetic code
  private SongGeneticCode code;

  //----------------------------- generateMovement --------------------------------
  //  Generates and returns a movement
  //---------------------------------------------------------------------------
  private Pattern generateMovement() {
      int numParts = code.BasicStructureChrom.getNextAlleleValue()%5+1;
      Queue q = new LinkedList();
      Verse v;

      //Generate parts
      for (int i=0; i<numParts; i++) {
        numMelodies = (code.BasicStructureChrom.getNextAlleleValue()%2)+1;

        v = new Verse(beatType, numBeats, tempo,
                       bassVoice, rythymVoice, leadVoices[1],
                       leadVoices[2], leadVoices[0], code, trans, numMelodies);
        q.offer(v);
      }

      //add verse and chorus to the movement and return it
      Pattern movement = new Pattern();
      for (int i=0; i<numParts; i++) { v = (Verse)q.poll(); movement.add(v.pattern); }
      //movement.add(v2.pattern);
      return movement;

  }
 
  //----------------------------- generateSong --------------------------------
  //  Generates a song and saves it to the specified file
  //  Parameters: File to Write Song to
  //---------------------------------------------------------------------------
  public void generateSong(File filePR) {
      int numMovements= code.BasicStructureChrom.getNextAlleleValue()%3+1;

      //Create a song pattern and add each part to the pattern
      Pattern song = new Pattern();

      //for each movement generate it and add it to the song
      for(int i=0; i<numMovements; i++) song.add(generateMovement(), 2);

      //Generate ending and add it to the song
      Ending ending = new Ending(leadVoices[1], bassVoice, leadVoices[0],
                          rythymVoice, trans);
      song.add(ending.pattern);
      
      //Create player and save the song
      Player player = new Player();
      player.play(ending.pattern);
      try {
        player.saveMidi(song, filePR);
      } catch(Exception e) {
        System.out.println("Exception: "+ e.getMessage());
      }
  }//generateSong()

  //----------------------------- pickLeadVoice --------------------------------
  //  Returns a lead voice based on the Basic Structure Chromosome
  //---------------------------------------------------------------------------
  private String pickLeadVoice() {
      String voice = new String();
      int r = code.BasicStructureChrom.getNextAlleleValue()%8;
      if (r==0) voice = "GUITAR";
      if (r==1) voice = "PIANO";
      if (r==2) voice = "DRAWBAR_ORGAN";
      if (r==3) voice = "FLUTE";
      if (r==4) voice = "ORCHESTRAL_STRINGS";
      if (r==5) voice = "BANJO";
      if (r==6) voice = "XYLOPHONE";
      if (r==7) voice = "SITAR";
      return voice;
  }

  //----------------------------- pickBassVoice --------------------------------
  //  Returns a bass voice based on the Basic Structure Chromosome
  //---------------------------------------------------------------------------
  private String pickBassVoice() {
      int r = code.BasicStructureChrom.getNextAlleleValue()%6;
      String voice = new String();
      if (r==0) voice = "Slap_Bass_1";
      if (r==1) voice = "TUBA";
      if (r==2) voice = "TROMBONE";
      if (r==3) voice = "PIANO";
      if (r==4) voice = "CELLO";
      if (r==5) voice = "TIMPANI";
      return voice;
  }

  //----------------------------- pickRhythmVoice --------------------------------
  //  Returns a rhythm voice based on the Rhythm Chromosome
  //---------------------------------------------------------------------------
  private String pickRhythmVoice() {
      int r = code.BasicStructureChrom.getNextAlleleValue()%5;
      String voice = new String();
      if (r==0) voice = "GUITAR";
      if (r==1) voice = "PIANO";
      if (r==2) voice = "ORCHESTRAL_STRINGS";
      if (r==3) voice = "KOTO";
      if (r==4) voice = "BANJO";
      return voice;
  }

  //----------------------------- pickBeatType --------------------------------
  //  Returns a beat type based on the Rhythm Chromosome
  //----------------------------------------------------------------------------
  public char pickBeatType() {
      int r = code.RhythmChrom.getNextAlleleValue()%3;
      if(r==0) return 'h';
      if(r==1) return 'i';
      return 'q';
  }

  //----------------------------- pickNumBeats --------------------------------
  //  Returns a number of beats based on the Rhythm Chromosome
  //---------------------------------------------------------------------------
  private int pickNumBeats() {
      //if statement adds a natural bias to common numbers of beats per
      //measure (4, 6, etc...)
      if (code.RhythmChrom.getNextAlleleValue()%3==0)
        return (code.RhythmChrom.getNextAlleleValue()%11)+2;
      else {

          if (beatType=='i') {
              if(code.RhythmChrom.getNextAlleleValue()%2==1) return 6;
              else return 8;
          }
          if (beatType=='q') {
              if(code.RhythmChrom.getNextAlleleValue()%2==1) return 4;
              else return 8;
          }
          if (beatType=='h') return 2;
      }
      return 4;
  }

  //----------------------------- pickTempo -----------------------------------
  //  Returns a tempo based on the Rhythm Chromosome
  //---------------------------------------------------------------------------
  public String pickTempo() {
      int r = code.BasicStructureChrom.getNextAlleleValue()%3;
      if (r==0) return "Presto";
      if (r==1) return "Pretissimo";
      return "allegro";
  }

  //----------------------------- Constructor --------------------------------
  //  Initializes the song's genetic code, tempo, time signature data,
  //  and voices
  //  Parameters: The Song's Genetic Code
  //-------------------------------------------------------------------------
  public Generator(SongGeneticCode chromosomePR) {
      code = chromosomePR;
      tempo = pickTempo();
      bassVoice = pickBassVoice();
      beatType = pickBeatType();
      numBeats = pickNumBeats();
      rythymVoice = pickRhythmVoice();
      leadVoices[2] = pickLeadVoice();
      leadVoices[1] = pickLeadVoice();
      leadVoices[0] = pickLeadVoice();
      trans = code.BasicStructureChrom.getNextAlleleValue()%11;
  }//Constructor

}//Generator

  


