import org.jfugue.*;

//----------------------------- Class::HarmonizingMelody --------------------------------
public class HarmonizingMelody {

    //Global Variables
    public Pattern pattern = new Pattern();  //

    //Note Durations
    private String[] durations = new String[15500];
    
    //Note Pitches
    private String[] pitches = new String[15500];
    
    //Notes to Harmonize With
    private String[] notesToHarmonizeWith = new String[15500];
    
    //Number of Notes
    public int numNotes;

    //Melody Chromosome
    private Chromosome chromosome;

    //Music Toolbox
    private MusicToolbox mtoolbox = new MusicToolbox();

    //----------------------------- Constructor --------------------------------
    //  Generates a second melody that harmonizes with the melody parameter
    //  Parameters: Melody, Chord Progression, Beat Type, Number of Beats,
    //              Number of Chords, Voice, Number of Chord Loops,
    //              Melody Chromosome
    //---------------------------------------------------------------------------
    public HarmonizingMelody(Melody melodyPR, String[] chordProgPR, char beatTypePR,
                             int numBeatsPR, int numChordsPR, String voicePR,
                             int numChordLoopsPR, Chromosome chromosomePR) {
        chromosome = chromosomePR;
        generateDurations(melodyPR.getDurations(), melodyPR.getNumNotes(), melodyPR.getNotes());
        generatePitches(chordProgPR, beatTypePR, numBeatsPR, numChordsPR);
        generatePattern(voicePR, numChordLoopsPR);
    }//Melody

    //----------------------------- generateDurations --------------------------
    //  Generates the durations of all pitches in the harmonizing melody
    //  Parameters: First Melody's Durations, Number of pitches,
    //              First Melody's pitches
    //--------------------------------------------------------------------------
    private void generateDurations(String[] durationsPR, int numNotesPR, String[] notesPR) {
        int notesLeft;
        
        int alleleValue = chromosome.getNextAlleleValue()%4;
        String duration = "i";
        int dur16=0;
        numNotes=0;
        int playShortNotes;
        
        for (int i=0; i<numNotesPR; i++)  {

          //use chromosome to decide if the next note will be longer or
          //shorter than the note it is to harmonize with
          playShortNotes = chromosome.getNextAlleleValue()%2;
         
          if (playShortNotes>0 || i+5>=numNotesPR) {
            //if the next note will be shorter than the note it is to 
            //harmonize with, pick a series of notes that will fit 
            //inside the note to harmonize with
            notesLeft=mtoolbox.getNum16ths(durationsPR[i].charAt(0));
            while(notesLeft>0) {

              do {
                if (alleleValue==0) { duration = "w"; dur16 = 16; }
                if (alleleValue==1) { duration = "h"; dur16 = 8;  }
                if (alleleValue==2) { duration = "q"; dur16 = 4;  }
                if (alleleValue==3) { duration = "i"; dur16 = 2;  }
                if (alleleValue==4) { duration = "s"; dur16 = 1;  }
                
                alleleValue = chromosome.getNextAlleleValue()%4;
              } while((notesLeft-dur16)<0);

              notesLeft = notesLeft-dur16;
              
              durations[numNotes] = duration;
              notesToHarmonizeWith[numNotes] = notesPR[i];
              numNotes++;
            }//while
          } else {
             int noteCount = 1;
             dur16=mtoolbox.getNum16ths(durationsPR[i].charAt(0));
             int oldi = i-1;

             //add the note durations of notes to harmonize with to dur16
             //until dur16 is 8, 4, or a number greater than 16
             while(dur16!=8 && dur16!=4 && dur16<16) {
                 i++;
                 noteCount++;
                 dur16 += mtoolbox.getNum16ths(durationsPR[i].charAt(0));
             }

             //set the duration string to a duration equal to dur16
             if(dur16<=16) {
               if(dur16==16) duration = "w";
               if(dur16==8)  duration = "h";
               if(dur16==4)  duration = "q";
               durations[numNotes] = duration;
               notesToHarmonizeWith[numNotes] = notesPR[i];
               numNotes++;
            } else {
              i=oldi;
            }//else failed to generate a longer note that will harmonize
          }//else next note should be longer than note to harmonize with
        }//for each note the melody must harmonize with
    }//generateDurations

    //----------------------------- getNextPitch -------------------------------
    //  Returns the next note given the currnt note, current chord,
    //  and a note to harmonize with.
    //  Parameters: Current Pitch, Current Chord, Pitch to Harmonize With
    //--------------------------------------------------------------------------
    private String getNextPitch(String cPitchPR, String chordPR, String pitchToHarmonizeWithPR) {
        String scale;
        int isArpeggioNote;

        //decide if next pitch will be based on a pentatonic mode or
        //a major mode
        int s = chromosome.getNextAlleleValue()%2;
        if (s==0) scale = "pentatonic";
        else scale = "major";

        //decide if next note will be an arpeggio
        if(mtoolbox.isArpeggio(chordPR.charAt(0), pitchToHarmonizeWithPR.charAt(0))) isArpeggioNote = chromosome.getNextAlleleValue()%2;
        else isArpeggioNote = 1;

        //if note is an arpeggio then return a note of the current chord
        //else return either the next or previous note in the scale
        if (isArpeggioNote==1) return mtoolbox.getArpeggioNote(chordPR, chromosome) + "5";
        else {
            int walkDown = chromosome.getNextAlleleValue()%2;
            if (walkDown==0) return mtoolbox.getNextPitchInScale(cPitchPR, scale);
            else return mtoolbox.getPrevPitchInScale(cPitchPR, scale);
        }//else

    }//getNextPitch

    //----------------------------- generatePitches --------------------------
    //  Generates pitches for harmonizing melody
    //  Parameters: Chord Progression, Beat Type, Number of Beats,
    //              Number of Chords
    //--------------------------------------------------------------------------
    private void generatePitches(String[] chordProgPR, char beatTypePR, int numBeatsPR,
                               int numChordsPR) {
        int notesLeft;
        int noteIndex = 0;       

        for (int i=0; i<numChordsPR; i++) {
           notesLeft=mtoolbox.getNum16ths(beatTypePR, numBeatsPR);

           while(notesLeft>0 && durations[noteIndex]!=null) {
                //if duration of next note is a whole or half note
                //then make it an arpeggio
                if(durations[noteIndex].equals("w")) {
                    notesLeft -= 16;
                    pitches[noteIndex] = mtoolbox.getArpeggioNote(chordProgPR[i], chromosome) +"5";
                } else if(durations[noteIndex].equals("h")) {
                    notesLeft -= 8;
                    pitches[noteIndex] = mtoolbox.getArpeggioNote(chordProgPR[i], chromosome) +"5";
                }else {
                    //if duration of next note is not a half or whole note
                    //then set pitches[noteIndex] to the next note
                    if(durations[noteIndex].equals("q")) notesLeft -= 4;
                    if(durations[noteIndex].equals("i")) notesLeft -= 2;
                    if(durations[noteIndex].equals("s")) notesLeft -= 1;
                    if(noteIndex>0 && notesToHarmonizeWith[noteIndex]!=null) pitches[noteIndex] = getNextPitch(pitches[noteIndex-1], chordProgPR[i], notesToHarmonizeWith[noteIndex]);
                    else pitches[noteIndex] = mtoolbox.getArpeggioNote(chordProgPR[i], chromosome) +"5";
                }//else
                noteIndex++;
            }//while there are notes in the measure
        }//for each chord in the progression
    }//generatePitches


    //----------------------------- generatePattern --------------------------
    //  Generates the melody's pattern
    //  Parameters: Voice, Numbr of Chord Loops
    //--------------------------------------------------------------------------
    private void generatePattern(String voicePR, int numChordLoopsPR) {
        //add pitches and durations to the melody string
        String mString = "V4 I[" + voicePR + "] ";
        for (int i=0; i<numNotes; i++) {
            mString += pitches[i] + durations[i] + " ";
        }

        //use string to generate the pattern
        pattern = new Pattern(mString);
        pattern.repeat(numChordLoopsPR);
    }//generateMelodyPattern

}//HarmonizingMelody
