import java.util.*;
import java.io.*;

//----------------------------- Class:Population --------------------------
public class Population {

    //Global Variables
    private Queue pop = new LinkedList();
    private final int maxPopSize = 50;  //population size

    //Number of alleles for each chromosome
    private final int numOfArpeggioAlleles =100;
    private final int numOfMelodyAlleles = 480;
    private final int numOfBasicStructureAlleles = 42;
    private final int numOfRhythmAlleles =16;

    private final int initialPopSize = 10; //size of initial population

    //----------------------------- Constructor --------------------------------
    public Population() { }

    //----------------------------- getRandomCode -----------------------------
    //  Returns a randomly generate chromosome
    //  Parameters:  Number of Alleles
    //-------------------------------------------------------------------------
    private Chromosome getRandomCode(int numOfAllelesPR) {
         Queue genCode = new LinkedList();
         Random r = new Random();

         for (int j=0; j<numOfAllelesPR; j++) {
             genCode.offer(r.nextInt(20));
         }//for each element in member's alleleQ

         Chromosome s = new Chromosome(genCode);
         return s;
    }

    //----------------------------- offerMember --------------------------------
    //  If population size is less than max population size then add genetic
    //  code to population, else delete the oldest half of the population and
    //  add genetic code
    //  Parameters:  Song Genetic Code
    //-------------------------------------------------------------------------
    public void offerMember(SongGeneticCode songGeneticCodePR) {
        if (pop.size()<=maxPopSize) {
            //if the population is smaller than it's max size
            //add the member to the population
            pop.offer(songGeneticCodePR);
        } else {
           //else remove the oldest half of the population
           //and add the member to the population
           for(int i=0; i<(maxPopSize/2); i++) pop.poll();
           pop.offer(songGeneticCodePR);
        }
    }

    //----------------------------- getCode -----------------------------------
    //  If population size is smaller than initial population size then
    //  return random genetic code else cross two members of the population
    //  and return their child
    //-------------------------------------------------------------------------
    public SongGeneticCode getCode() {
        if (pop.size()<initialPopSize) {
          return new SongGeneticCode(getRandomCode(numOfBasicStructureAlleles),
                                  getRandomCode(numOfRhythmAlleles),
                                  getRandomCode(numOfMelodyAlleles),
                                  getRandomCode(numOfArpeggioAlleles));
        } else {
          while(true) {
            SongGeneticCode rvalue = new SongGeneticCode();
            SongGeneticCode sgc1 = selectMember();
            SongGeneticCode sgc2 = selectMember();
           
            if (sgc1!=sgc2) {
                rvalue.ArpeggioChrom = crossover(sgc1.ArpeggioChrom, sgc2.ArpeggioChrom);
                rvalue.MelodyChrom = crossover(sgc1.MelodyChrom, sgc2.MelodyChrom);
                rvalue.RhythmChrom = crossover(sgc1.RhythmChrom, sgc2.RhythmChrom);
                rvalue.BasicStructureChrom = crossover(sgc1.BasicStructureChrom, sgc2.BasicStructureChrom);
                return rvalue;
            }//if two diffferent genetic codes were selected
          }//while selecting genetic codes
        }
    }

    //-----------------------------  -----------------------------------
    //  Returns the mutated crossover of two chromosomes
    //  Parameters: Chromosome one, Chromosome 2
    //-------------------------------------------------------------------------
    private Chromosome crossover(Chromosome chromosome1PR, Chromosome chromosome2PR) {
        //initialize variables
        Random r = new Random();
        int numOfAllelesPR=chromosome1PR.alleleQ.size();
        Queue c1Q = clone(chromosome1PR.alleleQ);
        Queue c2Q = clone(chromosome2PR.alleleQ);
        Queue q = new LinkedList();

        //determine the crossover point
        int crossPoint = r.nextInt(numOfAllelesPR/2)+(numOfAllelesPR/4)+1;

        //add the first part of chromosome 1 to the first
        //part of the new chromosome
        for (int i=0; i<crossPoint; i++) {
            q.offer(c1Q.poll());
            c2Q.remove();
        }

        //add the second part of chromosome 2 to the second
        //part of the new chromosome
        for (int i=crossPoint; i<numOfAllelesPR; i++) {
            q.offer(c2Q.poll());
        }

        //mutate and return the new chromosome
        return mutate(new Chromosome(q));
    }

    //----------------------------- mutate -----------------------------------
    //  Returns mutated chromosome
    //  Parameters: Chromosome
    //-------------------------------------------------------------------------
    private Chromosome mutate(Chromosome chromosomePR) {
        Queue q = chromosomePR.alleleQ;
        Random r = new Random();

        //for each allele in chromosome there is a 1/1000
        //chance of mutation
        for(int i=0; i<q.size(); i++) {
            int p = (Integer)q.poll();
            if (r.nextInt(1000)==1) q.offer(r.nextInt(20));
            else q.offer(p);
        }//for each allele

        return new Chromosome(q);
    }

    //----------------------------- clone -----------------------------------
    //  Returns a clone of a queue
    //  Parameters: Queue
    //-------------------------------------------------------------------------
    private Queue clone(Queue qPR) {
        LinkedList l = new LinkedList();
        l = (LinkedList)qPR;
        Queue r = (Queue)l.clone();
        return r;
    }

    //----------------------------- selectMember -------------------------------
    //  Uses roulette wheel slection to pick a member of the population
    //-------------------------------------------------------------------------
    private SongGeneticCode selectMember() {
      
        int sum=0;
        double sumProb=0;
        Queue myGen = pop;
        SongGeneticCode temp = (SongGeneticCode)myGen.peek();
        SongGeneticCode tempNext;

        //sum the fitness of all members of the population
        for (int i=pop.size()-1; i>0; i--) {
             temp = (SongGeneticCode)myGen.poll();
             sum += temp.fitness;
             myGen.offer(temp);
        }//for all members

        //calculate each members probability of selection
        //and sum the probabilities
        for (int i=pop.size()-1; i>0; i--) {
             temp = (SongGeneticCode)pop.poll();
             temp.probability = sumProb + ((double)temp.fitness/(double)sum);
             pop.offer(temp);
             sumProb += temp.probability;
        }//for all members

        
        myGen = clone(pop);
        

        //spin the wheel and select the member
        double num = Math.random();
        for (int i=pop.size()-1; i>0; i--) {
             temp = (SongGeneticCode)myGen.poll();
             tempNext = (SongGeneticCode)myGen.peek();
             if (num>temp.probability && num<tempNext.probability) {
                 return temp;
             }
        }//for all members

        return temp;
    }

    //----------------------------- save --------------------------------------
    //  Save population to g.dat
    //-------------------------------------------------------------------------
    public void save() {
      try {
        File f = new File("g.dat");
        f.createNewFile();
        Writer out = new PrintWriter(f);
        Queue myGen = clone(pop);
        Chromosome myChrom;
        Queue myAlleleQ;
        for(int i=0; i<pop.size(); i++) {
            SongGeneticCode sgc = (SongGeneticCode)myGen.poll();
            for (int chromosomePR=0; chromosomePR<4; chromosomePR++) {
                myChrom = sgc.ArpeggioChrom;
                if (chromosomePR==1) myChrom = sgc.BasicStructureChrom;
                if (chromosomePR==2) myChrom = sgc.MelodyChrom;
                if (chromosomePR==3) myChrom = sgc.RhythmChrom;
                myAlleleQ = clone(myChrom.alleleQ);
                for (int j=0; j<myChrom.alleleQ.size(); j++) {
                    out.write((Integer)myAlleleQ.poll());
                }//for each allele
            }//for each chromosome

            out.write(sgc.fitness);
        }//for each member of pop
        out.flush();

      } catch(Exception e) { System.out.println(e.toString()); }
    }

    //----------------------------- load --------------------------------------
    //  Load population from g.dat
    //-------------------------------------------------------------------------
    public void load() {
      try {
        Queue q = new LinkedList();
        int tmp = 0;
        SongGeneticCode sgc = new SongGeneticCode();
        FileInputStream fis = new FileInputStream("g.dat");
        for (;;) {
          int numOfAllelesPR = numOfArpeggioAlleles;
          for (int chromosomePR=0; chromosomePR<4; chromosomePR++) {
            for(int i=0; i<numOfAllelesPR; i++) {
              tmp = fis.read();
              if (tmp==-1) break;
              q.offer(tmp);
            }
            if (tmp==-1) break;
            if (chromosomePR==0) {
                sgc.ArpeggioChrom = new Chromosome(q);
                numOfAllelesPR = numOfBasicStructureAlleles;
            }
            if (chromosomePR==1) {
                sgc.BasicStructureChrom = new Chromosome(q);
                numOfAllelesPR = numOfMelodyAlleles;
            }
            if (chromosomePR==2) {
                sgc.MelodyChrom = new Chromosome(q);
                numOfAllelesPR = numOfRhythmAlleles;
            }
            if (chromosomePR==3) sgc.RhythmChrom = new Chromosome(q);
            q = new LinkedList();
          }//for each chromosome
          if (tmp==-1) break;
          sgc.fitness = fis.read();
          pop.offer(sgc);
          sgc = new SongGeneticCode();
        }//for each member of the population

      } catch(Exception e) { System.out.println(e.toString()); }
    }

}//Population
