import org.jfugue.*;

//----------------------------- Class:Rhythm --------------------------------
public class Rhythm {

    //Chord progression variables
    private String[] chordProg = new String[28];  //Chord Progression
    private int numChords;                        //Number of Chords

    //Time signature variables
    private char beatType;
    private int  numBeats;

    //Other global variables
    private int numrhythmNotes;                   //Number of Notes in Rhythm
    private String voice = new String();          //Rhythm Voice
    private String[] rhythm = new String[225];    //Rhythm String
    private Chromosome chromosome;                //Rhythm Chromosome
    public Pattern pattern = new Pattern();
    private MusicToolbox mtoolbox = new MusicToolbox();

    //----------------------------- Constructor -----------------------------
    //  Generates a rhythm pattern
    //  Parameters:  Number of Chords, Beat Type, Number of Beats,
    //               Chord Progression, Rhythm Voice, Number of Chord Loops,
    //               Tempo, Chromosome
    //-------------------------------------------------------------------------
    public Rhythm(int numChordsPR, char beatTypePR, int numBeatsPR,
                  String[] chordProgPR, String rhythmVoicePR,
                  int numChordLoopsPR, String tempoPR, Chromosome chromosomePR) {
      numChords = numChordsPR;
      chromosome = chromosomePR;
      numBeats = numBeatsPR;
      chordProg = chordProgPR;
      voice = rhythmVoicePR;
      beatType = beatTypePR;
      generatRhythm();
      generatRhythmPat(numChordLoopsPR, tempoPR);
    }//Constructor

    //----------------------------- generateRhythm -----------------------------
    //  Generates the note durations of the rhythm
    //-------------------------------------------------------------------------
    private void generatRhythm() {
      String duration = new String(); //duration of current note
      int dur16 = 0; //duration of current note in # of 16th pitches
      numrhythmNotes = 0;

      //initilize # of pitches left in measure
      //and determine duration of first note
      int notesLeft = mtoolbox.getNum16ths(beatType, numBeats);
      int i, k;
      i = chromosome.getNextAlleleValue()%4;

      //for each 16th note in the measure
      //pick a new note based on genetic algorithm
      for (k=0; notesLeft>0; k++) {

        do {
          if (i==0) { duration = "w"; dur16 = 16; }
          if (i==1) { duration = "h"; dur16 = 8;  }
          if (i==2) { duration = "q"; dur16 = 4;  }
          if (i==3) { duration = "i"; dur16 = 2;  }
          i = chromosome.getNextAlleleValue()%4;
        } while((notesLeft-dur16)<0);

        notesLeft = notesLeft-dur16;
        rhythm[numrhythmNotes] = duration;
        numrhythmNotes++;
      }//for each note in measure

      //Loop rhythm for all chords in the progression
      notesLeft = mtoolbox.getNum16ths(beatType, numBeats);
      for (int j=0; j<(numChords-1); j++) {
        for (k=0; notesLeft>0; k++) {
          if(rhythm[k].equals("w")) notesLeft -= 16;
          if(rhythm[k].equals("h")) notesLeft -= 8;
          if(rhythm[k].equals("q")) notesLeft -= 4;
          if(rhythm[k].equals("i")) notesLeft -= 2;
          if(rhythm[k].equals("s")) notesLeft -= 1;
          rhythm[numrhythmNotes+k] = rhythm[k];
        }
        numrhythmNotes += k;
        notesLeft = mtoolbox.getNum16ths(beatType, numBeats);
      }//for each chord loop the durations  of the rhythm

  }//generateRythym

  //----------------------------- generateRhythmPat -----------------------------
  //  Generates the rhythm's pattern
  //  Parameters: Number of Chord Loops, Tempo
  //-------------------------------------------------------------------------
  private void generatRhythmPat(int numChordLoopsPR, String tempoPR) {
      //add tempo and voice info to pattern sring
      String rString = "T[";
      int j=0;
      int notesLeft = mtoolbox.getNum16ths(beatType, numBeats);
      rString += tempoPR + "] V0 I[" + voice + "] ";

      //For each note in the rhythm add it to the pattern string
      for(int i=0; i<numrhythmNotes; i++) {
        rString += chordProg[j] + rhythm[i] + " ";
        if (rhythm[i].equals("w")) notesLeft -= 16;
        if (rhythm[i].equals("h")) notesLeft -= 8;
        if (rhythm[i].equals("q")) notesLeft -= 4;
        if (rhythm[i].equals("i")) notesLeft -= 2;
        if (rhythm[i].equals("s")) notesLeft -= 1;
        if(notesLeft==0) {
          j++;
          notesLeft = mtoolbox.getNum16ths(beatType, numBeats);
        }
      }

      //Create pattern using pattern string
      pattern = new Pattern(rString);
      pattern.repeat(numChordLoopsPR);

      //adjust numChords for the # of chord loops
      numChords = numChords*numChordLoopsPR;
  }//generateRhythmPat()

  //----------------------------- getRhythm -----------------------------
  //  Returns the rhythm string
  //-------------------------------------------------------------------------
  public String[] getRhythm() {
      return rhythm;
  }//getRhythm()

  //----------------------------- getNumNotes -----------------------------
  //  Returns the number of notes in the rhythm
  //  Parameters:  Number of Alleles
  //-------------------------------------------------------------------------
  public int getNumNotes() {
      return numrhythmNotes;
  }//getNumNotes()

}//Rhythm
