import org.jfugue.extras.IntervalPatternTransformer;
import org.jfugue.*;

//----------------------------- Class:Verse -----------------------------
public class Verse {

  public Pattern pattern = new Pattern();
  private char beatType;
  private int numBeats;
  private int trans;
  private int numChordLoops;
  private int numMelodies;
  private String tempo = new String();
  private String bassVoice = new String();
  private String rhythmVoice = new String();
  private String melodyVoice = new String();
  private String melodyVoice2 = new String();
  private String harmonyVoice = new String();
  private int numChords;
  private SongGeneticCode code;

  Pattern rhythmPat = new Pattern();
  public String[] chordProg = new String[28];

  //----------------------------- getChord --------------------------------
  //  Returns a chord based on the Basic Structure Chromosome
  //-------------------------------------------------------------------------
  private String getChord() {
      int i = code.BasicStructureChrom.getNextAlleleValue()%6;
      String chord = new String();
      if (i==0) chord = "C4maj";
      if (i==1) chord = "D4min";
      if (i==2) chord = "E4min";
      if (i==3) chord = "F4maj";
      if (i==4) chord = "G4maj";
      if (i==5) chord = "A4min";
      return chord;
  }//getChord()

  //----------------------------- getNextChord --------------------------------
  //  Ruturns the next chord based on the current chord and
  //  Basic Structure Chromosome
  //  Parameters:  Current Chord
  //-------------------------------------------------------------------------
  private String getNextChord(String cChordPR) {
      String chord = new String();
      cChordPR = cChordPR.substring(0, 5);
      //Use genetic code to select the next chord in the progression
      int i = code.BasicStructureChrom.getNextAlleleValue()%6;
      if (cChordPR.equals("C4maj")) {
        if (i==0) chord = "D4min";
        if (i==1) chord = "E4min";
        if (i==2) chord = "F4maj";
        if (i==3) chord = "G4maj";
        if (i==4) chord = "A4min";
        if (i==5) chord = "C4maj";
      }
      if (cChordPR.equals("D4min")) {
        if (i==0) chord = "C4maj";
        if (i==1) chord = "E4min";
        if (i==2) chord = "F4maj";
        if (i==3) chord = "G4maj";
        if (i==4) chord = "A4min";
        if (i==5) chord = "D4min";
      }
      if (cChordPR.equals("E4min")) {
        if (i==0) chord = "D4min";
        if (i==1) chord = "C4maj";
        if (i==2) chord = "F4maj";
        if (i==3) chord = "G4maj";
        if (i==4) chord = "A4min";
        if (i==5) chord = "E4min";
      }
      if (cChordPR.equals("F4maj")) {
        if (i==0) chord = "D4min";
        if (i==1) chord = "E4min";
        if (i==2) chord = "C4maj";
        if (i==3) chord = "G4maj";
        if (i==4) chord = "A4min";
        if (i==5) chord = "F4maj";
      }
      if (cChordPR.equals("G4maj")) {
        if (i==0) chord = "D4min";
        if (i==1) chord = "E4min";
        if (i==2) chord = "F4maj";
        if (i==3) chord = "C4maj";
        if (i==4) chord = "A4min";
        if (i==5) chord = "G4maj";
      }
      if (cChordPR.equals("A4min")) {
        if (i==0) chord = "D4min";
        if (i==1) chord = "E4min";
        if (i==2) chord = "F4maj";
        if (i==3) chord = "G4maj";
        if (i==4) chord = "C4maj";
        if (i==5) chord = "A4min";
      }
      if ((code.BasicStructureChrom.getNextAlleleValue()%3)==0) return chord + "14";
      else if ((code.BasicStructureChrom.getNextAlleleValue()%3)==0) return chord + "9";
      else return chord;
  }//getNextChord()


  //----------------------------- generateProgression --------------------------------
  //  Generates the verse's chord progression
  //-------------------------------------------------------------------------
  private void generateProgression() {
      int i;
      chordProg[0] = getChord();
      for (i=1; i<numChords; i++) chordProg[i] = getNextChord(chordProg[i-1]);
      chordProg[i] = "eoc";
  }//generateProgression()

  //----------------------------- generateVerse --------------------------------
  //  Generates all the parts in the verse.
  //-------------------------------------------------------------------------
  public void generateVerse() {
      //Generate the progression, beat, rythym, harmonies, and melodies
      int numHarmonies = code.BasicStructureChrom.getNextAlleleValue()%3;
      Harmony h[] = new Harmony[numHarmonies];
      generateProgression();
      Beat b = new Beat(beatType, numBeats, numChords, numChordLoops, code.RhythmChrom);
      
      Rhythm r = new Rhythm(numChords, beatType, numBeats, chordProg,
                            rhythmVoice, numChordLoops, tempo, code.RhythmChrom);

      Harmony bl = new Harmony(numChordLoops, bassVoice,
                               r.getNumNotes(), chordProg, beatType,
                               numBeats, r.getRhythm(), "3", 2, code);

      for(int i=0; i<numHarmonies; i++)
          h[i] = new Harmony(numChordLoops, harmonyVoice, r.getNumNotes(),
                             chordProg, beatType, numBeats, r.getRhythm(), "5",
                             i+5, code);

      Melody m = new Melody(r, chordProg, beatType, numBeats, numChords,
                            melodyVoice, numChordLoops, code.MelodyChrom);

      HarmonizingMelody m2;
      
      //add parts of verse to pattern
      pattern = new Pattern();
      pattern.add(r.pattern, 1);
      pattern.add(bl.pattern, 1);
      for(int i=0; i<numHarmonies; i++) pattern.add(h[i].pattern, 1);
      pattern.add(m.pattern, 1);
      if (numMelodies==2) {
          m2 = new HarmonizingMelody(m, chordProg, beatType, numBeats,
                                     numChords, melodyVoice2, numChordLoops,
                                     code.MelodyChrom);
          pattern.add(m2.pattern, 1);
      }
      //transpose song and add the beat
      IntervalPatternTransformer transposer = new IntervalPatternTransformer(trans);
      pattern = transposer.transform(pattern);
      pattern.add(b.pattern, 1);
     
  }//generateVerse()

  //----------------------------- Constructor --------------------------------
  //  Initializes global variables and generates verse
  //  Parameters:  Beat Type, Number of Beats, Tempo, Bassline Voice,
  //               Rhythm Voice, Melody Voice, Second Melody Voice,
  //               Harmony Voice, Song's Genetic Code,
  //               Amount to Transpose Song, Number of Melodies
  //-------------------------------------------------------------------------
  public Verse(char beatTypePR, int numBeatsPR, String tempoPR,
               String basslineVoicePR, String rhythmVoicePR, String melodyVoicePR,
               String melody2VoicePR, String harmonyVoicePR, SongGeneticCode codePR,
               int transPR, int numMelodiesPR) {
      //initialize global variables and generate verse
      code = codePR;
      melodyVoice2 = melody2VoicePR;
      numChords = (code.BasicStructureChrom.getNextAlleleValue()%6)+4;
      trans = transPR;
      numChordLoops = (code.BasicStructureChrom.getNextAlleleValue()%2)+1;
      beatType = beatTypePR;
      numBeats = numBeatsPR;
      tempo = tempoPR;
      bassVoice = basslineVoicePR;
      rhythmVoice = rhythmVoicePR;
      melodyVoice = melodyVoicePR;
      harmonyVoice = harmonyVoicePR;
      numMelodies = numMelodiesPR;
      generateVerse();
    }//Constructor

}//Verse
